C===================================================================
C
      PROGRAM ELBECON
C
C
C  This program solves the two dimensional (EL)asticiy problem
C  using the (B)oundary (E)lement method with (CON)stant
C  boundary elements for domain with multiple boundaries
C
      USE MSIMSL
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*15 INPUTFILE,OUTPUTFILE
C
C  Set the maximum dimensions
C
      PARAMETER (N=44) 
      PARAMETER (IN=3)
      PARAMETER (NB=1)
C
C  N= Number of boundary elements equal to number of boundary 
C     nodes
C
C IN= Number of internal points where the function u is calculated
C
C NB= Number of boundaries of the multiple boundary domain
C
      COMMON EL,GL,PN,ALAMDA
      DIMENSION NL(NB),KCODE(N)
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),G(2*N,2*N),H(2*N,2*N)
      DIMENSION UB(N),VB(N),TXB(N),TYB(N),A(2*N,2*N),R(2*N)
      DIMENSION XIN(IN),YIN(IN),UIN(IN),VIN(IN)
      DIMENSION SXIN(IN),SYIN(IN),SXYIN(IN),SXB(N),SYB(N),SXYB(N)
C
C  Read the names and open the input and output files
C
      WRITE (*,'(A)')' Name of the INPUTFILE (max.15 characters)'
      READ  (*,'(A)') INPUTFILE
      WRITE (*,'(A)')' Name of the OUTPUTFILE (max.15 characters)'
      READ  (*,'(A)') OUTPUTFILE
      OPEN (1, FILE=INPUTFILE)
      OPEN (2, FILE=OUTPUTFILE)
C
C  Read data from INPUTFILE
C
      CALL INPUTEL(XL,YL,XIN,YIN,KCODE,IPLANE,NL,UB,VB,N,IN,NB)
C
C  Compute the G matrix
C
      CALL GMATREL(XL,YL,XM,YM,G,N,NL,NB)
C
C  Compute the H matrix
C
      CALL HMATREL(XL,YL,XM,YM,H,N,NL,NB)
C
C  Form the system of equations AX=B
C
      CALL ABMATREL(G,H,A,R,UB,VB,KCODE,N)
C
C  Solve the system of equations
C
      CALL SOLVEQ(A,R,N,LSING)
C
C  Form the vectors U and UN of all the boundary values
C
      CALL REORDEREL(R,UB,VB,TXB,TYB,KCODE,N)
C
C  Compute the values of the displacements at the internal points
C
      CALL UVINTER(XL,YL,XIN,YIN,UB,VB,TXB,TYB,UIN,VIN,N,IN,NL,NB)
C
C  Compute the values of the stresses at the internal points
C
      CALL STRESSIN(XL,YL,XIN,YIN,UB,VB,TXB,TYB,
     1                   SXIN,SYIN,SXYIN,N,IN,NL,NB)
C
C  Compute the values of the boundary stresses
C
      CALL STRESSB(XL,YL,UB,VB,TXB,TYB,SXB,SYB,SXYB,N,NL,NB)
C
C  Print the results in the OUTPUTFILE
C
      CALL OUTPUTEL(XM,YM,UB,VB,TXB,TYB,XIN,YIN,UIN,VIN,
     1              SXB,SYB,SXYB,SXIN,SYIN,SXYIN,N,IN)
C
C  Close input and output files
C
      CLOSE(1)
      CLOSE(2)
      STOP
      END
C
C
C====================================================================
C
      SUBROUTINE INPUTEL(XL,YL,XIN,YIN,KCODE,IPLANE,NL,UB,VB,N,IN,NB)
C
C
C  This subroutine reads the input data from the input file
C  and writes them in the output file
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON EL,GL,PN,ALAMDA
      CHARACTER*80 NAME,TITLE
      DIMENSION NL(NB),KCODE(N)
      DIMENSION UB(N),VB(N)
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN)
C
      WRITE(2,100)
  100 FORMAT('',76('*'))
      READ(1,'(A)') NAME
      WRITE(2,'(A)') NAME
C
C  Read the title of the program
C
      READ(1,'(A)')TITLE
      WRITE(2,'(A)')TITLE
C
      WRITE(2,200)N,IN,NB
  200 FORMAT(//'DATA'//2X,'NUMBER OF BOUNDARY ELEMENTS ='
     1      ,I3/2X,'NUMBER OF INTERNAL POINTS   =',I3/2X,
     1       'NUMBER OF BOUNDARIES        =',I3)
C
C  Read the type of the problem: IPLANE=0 for plane strain,
C                                IPLANE=1 for plane stress
C
      READ(1,*)IPLANE
      IF(IPLANE.EQ.0)THEN
      WRITE(2,700)
  700 FORMAT(/'  PLANE STRAIN PROBLEM'/)
      ELSE
      WRITE(2,450)
  450 FORMAT(/'  PLANE STRESS PROBLEM'/)
      ENDIF
C
C  Read the elastic constants
C
      READ(1,*) EL,PN
C
      WRITE(2,150) EL,PN
  150 FORMAT(2X,'ELASTIC CONSTANTS:','   ELASTIC MODULUS =',E10.4,
     1    '  POISSON RATIO =',F5.2/)
C
      IF(IPLANE.EQ.0)THEN
      EL=EL/(1.-PN**2)
      PN=PN/(1-PN)
      GL=EL/(2.*(1+PN))
      ALAMDA=PN*EL/(1-PN**2)
      ELSEIF(IPLANE.NE.0)THEN
      GL=EL/(2.*(1+PN))
      ALAMDA=PN*EL/(1-PN**2)
      ENDIF
C
C  Read the of the last element at each boundary
C
      READ(1,*)(NL(I), I=1,NB)
C
C  Read the coordinates of the extreme points of the boundary
C  elements XL,YL
C
      READ(1,*) (XL(I),YL(I),I=1,N)
C
C  Write the coordinates in the output file
C
      WRITE(2,300)
  300 FORMAT(//'  COORDINATES OF THE EXTREME POINTS OF THE BOUNDARY',
     1         ' ELEMENTS',//1X,'NODE',9X,'  XL',13X,'  YL'/)
      DO 20 I=1,N
   20 WRITE(2,400) I,XL(I),YL(I)
  400 FORMAT(2X,I3,2(3X,E14.5))
C
C  Read the boundary conditions and store in UB(I) and VB(I)
C
C    KCODE(I)=1 when u  and v  are prescribed
C    KCODE(I)=2 when tx and ty are prescribed
C    KCODE(I)=3 when u  and ty are prescribed
C    KCODE(I)=4 when tx and v  are prescribed
C
      READ(1,*) (KCODE(I),UB(I),VB(I),I=1,N)
C
C  Write the boundary conditions in the output file
C
      WRITE(2,500)
  500 FORMAT(//2X,'BOUNDARY CONDITIONS'//2X,'NODE',6X,'  KCODE',
     1 20X,'PRESCRIBED VALUES'/)
      DO 30 I=1,N
   30 WRITE(2,600) I,KCODE(I),UB(I), VB(I)
  600 FORMAT (2X,I3,9X,I3,8X,E14.5,8X,E14.5)
C
C  Read the coordinates of the internal points
C
      READ(1,*) (XIN(I),YIN(I),I=1,IN)
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE GMATREL (XL,YL,XM,YM,G,N,NL,NB)
C
C
C  This subroutine computes the elements of the G matrix
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON EL,GL,PN,ALAMDA
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N)
      DIMENSION NL(NB),G(2*N,2*N)
C
C  Compute the nodal coordinates and store them in the arrays XM 
C  and YM
C
      PI=ACOS(-1.)
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
      DO 10  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
   10 YM(I)=(YL(I)+YL(I+1))/2.
      IF(NB.LE.1) GOTO 40
      XM(NL(1))=(XL(NL(1))+XL(1))/2.
      YM(NL(1))=(YL(NL(1))+YL(1))/2.
      DO 15 K=2,NB
      XM(NL(K))=(XL(NL(K))+XL(NL(K-1)+1))/2.
   15 YM(NL(K))=(YL(NL(K))+YL(NL(K-1)+1))/2.
C
C  Compute the elements of matrix G
C
   40 DO 20 I=1,N
      X0=XM(I)
      Y0=YM(I)
      DO 20 J=1,N
      X1=XL(J)
      Y1=YL(J)
      X2=XL(J+1)
      Y2=YL(J+1)
      IF(NB.LE.1) GOTO 60
      IF(J.NE.NL(1))GOTO 50
            X2=XL(1)
            Y2=YL(1)
            GOTO 60
   50      DO 30 K=2,NB
          IF(J.NE.NL(K))GOTO 30
            X2=XL(NL(K-1)+1)
            Y2=YL(NL(K-1)+1)
            GOTO 60
   30       CONTINUE
   60 IF(I.NE.J)THEN
C
C	IFLAG=1: for the boundary nodes
C	IFLAG=2: for the internal points
C
      IFLAG=1
C
      CALL RLINTG(X0,Y0,X1,Y1,X2,Y2,RES11,RES12,RES22,IFLAG)
      G(2*I-1,2*J-1)=RES11
      G(2*I-1,2*J)=RES12
      G(2*I,2*J-1)=RES12
      G(2*I,2*J)=RES22
      ELSEIF(I.EQ.J)THEN
      DX=X2-X1
      DY=Y2-Y1
      SL=DSQRT(DX**2+DY**2)
      COSA=DX/SL
      SINA=DY/SL
      SPIG=SL/(8.*PI*GL)
C
	C3=(7.-PN)/2.
C
      RES11=-SPIG*((3.-PN)*(DLOG(SL/2.)-1.)-(1.+PN)*COSA**2+C3)
      RES12=SPIG*(1.+PN)*COSA*SINA
      RES22=-SPIG*((3.-PN)*(DLOG(SL/2.)-1.)-(1.+PN)*SINA**2+C3)
      G(2*I-1,2*J-1)=RES11
      G(2*I-1,2*J)=RES12
      G(2*I,2*J-1)=RES12
      G(2*I,2*J)=RES22
      ENDIF
   20 CONTINUE
      RETURN
      END
C
C
C================================================================
C
      SUBROUTINE RLINTG(X0,Y0,X1,Y1,X2,Y2,RES11,RES12,RES22,IFLAG)
C
C
C  This subroutine computes the off-diagonal elements of the 
C  matrix G
C
C  RA= The distance of the point O to the Gauss integration point 
C      on the boundary element
C
C  WG= The weights of the Gauss integration
C
C  XI= The coordinates of the Gauss integration points in the 
C      interval [-1,1]
C
C  XC,YC=The global coordinates of the Gauss integration points
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON EL,GL,PN,ALAMDA
      DIMENSION XC(4),YC(4),XI(4),WG(4)
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      BX=(X2+X1)/2.
      BY=(Y2+Y1)/2.
C
C  Compute the Jacobian
C
      SL=DSQRT(AX**2+AY**2)
C
C  Compute the line integral
C
      RES11=0.
      RES12=0.
      RES22=0.
C
      PIG=1./(8.*PI*GL)
      DO 30 I=1,4
      XC(I)=AX*XI(I)+BX
      YC(I)=AY*XI(I)+BY
      DX=XC(I)-X0
      DY=YC(I)-Y0
      RA=DSQRT(DX**2+DY**2)
      COSA=DX/RA
      SINA=DY/RA
C
	IF (IFLAG.EQ.1) THEN
	C3=(7.-PN)/2.
	ELSEIF (IFLAG.EQ.2) THEN
	C3=0.
	ENDIF
C
      RES11=RES11-PIG*((3.-PN)*DLOG(RA)-(1+PN)*COSA**2+C3)*WG(I)*SL
      RES12=RES12+PIG*(1.+PN)*COSA*SINA*WG(I)*SL
   30 RES22=RES22-PIG*((3.-PN)*DLOG(RA)-(1+PN)*SINA**2+C3)*WG(I)*SL
      RETURN
      END
C
C
C=================================================================
C
      SUBROUTINE HMATREL(XL,YL,XM,YM,H,N,NL,NB)
C
C
C  This subroutine computes the elements of the H matrix
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON EL,GL,PN,ALAMDA
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),NL(NB)
      DIMENSION H(2*N,2*N)
C
C  Compute the nodal coordinates and store them in the arrays XM 
C  and YM
C
      PI=ACOS(-1.)
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
      DO 10  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
   10 YM(I)=(YL(I)+YL(I+1))/2.
      IF(NB.LE.1) GOTO 40
      XM(NL(1))=(XL(NL(1))+XL(1))/2.
      YM(NL(1))=(YL(NL(1))+YL(1))/2.
      DO 15 K=2,NB
      XM(NL(K))=(XL(NL(K))+XL(NL(K-1)+1))/2.
   15 YM(NL(K))=(YL(NL(K))+YL(NL(K-1)+1))/2.
C
C  Compute the elements of matrix H
C
   40 DO 20 I=1,N
      X0=XM(I)
      Y0=YM(I)
      DO 20 J=1,N
      X1=XL(J)
      Y1=YL(J)
      X2=XL(J+1)
      Y2=YL(J+1)
      IF(NB.LE.1) GOTO 60
      IF(J.NE.NL(1))GOTO 50
            X2=XL(1)
            Y2=YL(1)
            GOTO 60
   50      DO 30 K=2,NB
          IF(J.NE.NL(K))GOTO 30
            X2=XL(NL(K-1)+1)
            Y2=YL(NL(K-1)+1)
            GOTO 60
   30       CONTINUE
   60 IF(I.NE.J)THEN
      CALL RLINTH(X0,Y0,X1,Y1,X2,Y2,RES11,RES12,RES21,RES22)
      H(2*I-1,2*J-1)=RES11
      H(2*I-1,2*J)=RES21
      H(2*I,2*J-1)=RES12
      H(2*I,2*J)=RES22
      ELSEIF (I.EQ.J) THEN
      H(2*I-1,2*J-1)=.5
      H(2*I-1,2*J)=0.
      H(2*I,2*J-1)=0.
      H(2*I,2*J)=.5
      ENDIF
   20 CONTINUE
      RETURN
      END
C
C
C================================================================
C
      SUBROUTINE RLINTH(X0,Y0,X1,Y1,X2,Y2,RES11,RES12,RES21,RES22)
C
C
C  This subroutine computes the off-diagonal elements of the 
C  matrix H
C
C  RA= The distance of the point O to the Gauss integration point 
C      on the boundary element
C
C  WG= The weights of the Gauss integration
C
C  XI= The coordinates of the Gauss integration points in the 
C      interval [-1,1]
C
C  XC,YC=The global coordinates of the Gauss integration points
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON EL,GL,PN,ALAMDA
      DIMENSION XC(4),YC(4),XI(4),WG(4)
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      BX=(X2+X1)/2.
      BY=(Y2+Y1)/2.
      SL=DSQRT(AX**2+AY**2)
      ENX=AY/SL
      ENY=-AX/SL
C
C  Compute the line integral
C
      RES11=0.
      RES12=0.
      RES21=0.
      RES22=0.
C
      A1=-(1.+PN)/(4.*PI)
      A2=(1.-PN)/(1.+PN)
      DO 30  I=1,4
      XC(I)=AX*XI(I)+BX
      YC(I)=AY*XI(I)+BY
      DX=XC(I)-X0
      DY=YC(I)-Y0
      RA=DSQRT(DX**2+DY**2)
      COSA=DX/RA
      SINA=DY/RA
      RN=COSA*ENX+SINA*ENY
      RT=SINA*ENX-COSA*ENY
      RES11=RES11+(A1/RA)*(A2+2.*COSA**2)*RN*WG(I)*SL
      RES21=RES21+(A1/RA)*(2.*COSA*SINA*RN+A2*RT)*WG(I)*SL
      RES12=RES12+(A1/RA)*(2.*COSA*SINA*RN-A2*RT)*WG(I)*SL
   30 RES22=RES22+(A1/RA)*(A2+2.*SINA**2)*RN*WG(I)*SL
      RETURN
      END
C
C
C=================================================================
C
      SUBROUTINE ABMATREL(G,H,A,R,UB,VB,KCODE,N)
C
C
C  This subroutine rearranges the matrices G and H and produces the
C  matrices A and R
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION G(2*N,2*N),H(2*N,2*N),A(2*N,2*N)
      DIMENSION UB(N),VB(N),KCODE(N),R(2*N)
C
C  Reorder the columns the system of equations and store them in A
C
      N2=2*N
      DO 20 I=1,N2
      R(I)=0.
      DO 20 J=1,N2
   20 A(I,J)=0.
      DO 70 J=1,N
      GOTO(30,40,50,60)KCODE(J)
   30 DO 35 I=1,N2
      A(I,2*J-1)=-G(I,2*J-1)
      A(I,2*J)=-G(I,2*J)
   35 R(I)=R(I)-H(I,2*J-1)*UB(J)-H(I,2*J)*VB(J)
      GOTO 70
   40 DO 45 I=1,N2
      A(I,2*J-1)=H(I,2*J-1)
      A(I,2*J)=H(I,2*J)
   45 R(I)=R(I)+G(I,2*J-1)*UB(J)+G(I,2*J)*VB(J)
      GOTO 70
   50 DO 55 I=1,N2
      A(I,2*J-1)=-G(I,2*J-1)
      A(I,2*J)=H(I,2*J)
   55 R(I)=R(I)-H(I,2*J-1)*UB(J)+G(I,2*J)*VB(J)
      GOTO 70
   60 DO 65 I=1,N2
      A(I,2*J-1)=H(I,2*J-1)
      A(I,2*J)=-G(I,2*J)
   65 R(I)=R(I)+G(I,2*J-1)*UB(J)-H(I,2*J)*VB(J)
   70 CONTINUE
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE SOLVEQ(A,R,N,LSING)
C
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(2*N,2*N),R(2*N)
      NN=2*N
      CALL LEQS(A,R,NN,LSING)
      IF(LSING.EQ.0)THEN
      WRITE(2,150)
  150 FORMAT(/,'',76('*')//2X'The system has been solved regularly'/)
      ELSEIF(LSING.EQ.1)THEN
      WRITE(2,170)
  170 FORMAT(/,'',76('*')//2X'The system is singular'/)
      ENDIF
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE LEQS(A,B,N,LSING)
C
C
C   This subroutine uses Gauss elimination to solve
C   a system of linear equations, [A]{X}={B}, where
C   A : One-dimensional array which contains the occasional row of
C       the two-dimensional array of the coefficients of the unknowns
C   B : One-dimensional array which contains the known coefficients
C   N : Integer denoting the number of the unknowns
C   LSING: Integer taking the values:
C         LSING = 0, if the system has been solved regularly
C         LSING = 1, if the system is singular
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(1)
C
      LSING=0
      JJ=-N
      DO 10 J=1,N
        JY=J+1
        JJ=JJ+N+1
        AMAX=0.0
        IHELP=JJ-J
      DO 20 I=J,N
        IJ=IHELP+I
        IF(ABS(AMAX)-ABS(A(IJ))) 30,20,20
   30      AMAX=A(IJ)
           IMAX=I
   20 CONTINUE
         IF(ABS(AMAX).EQ.0.) THEN
            LSING=1
            RETURN
         END IF
      I1=J+N*(J-2)
      IHELP=IMAX-J
      DO 40 K=J,N
        I1=I1+N
        I2=I1+IHELP
        ATEMP=A(I1)
        A(I1)=A(I2)
        A(I2)=ATEMP
   40 A(I1)=A(I1)/AMAX
      ATEMP=B(IMAX)
      B(IMAX)=B(J)
      B(J)=ATEMP/AMAX
         IF(J-N) 50,70,50
   50       IQS=N*(J-1)
            DO 10 IX=JY,N
              IXJ=IQS+IX
              IHELP=J-IX
              DO 60 JX=JY,N
                IJREF=N*(JX-1)+IX
                JJX=IJREF+IHELP
   60           A(IJREF)=A(IJREF)-(A(IXJ)*A(JJX))
   10 B(IX)=B(IX)-B(J)*A(IXJ)
   70    NY=N-1
      NN=N*N
      DO 80 J=1,NY
        I1=NN-J
        I2=N-J
        I3=N
      DO 80 K=1,J
        B(I2)=B(I2)-A(I1)*B(I3)
        I1=I1-N
   80   I3=I3-1
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE REORDEREL(R,UB,VB,TXB,TYB,KCODE,N)
C
C
C  This subroutine rearranges the arrays UB and VB 
C  in such a way that all boundary values of u and v
C  are stored in UB and VB, while all  boundary
C  values of tx and ty in TXB and TYB
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION UB(N),VB(N),R(2*N),TXB(N),TYB(N),KCODE(N)
C
      DO 50  I=1,N
      GOTO(10,20,30,40) KCODE(I)
   10 TXB(I)=R(2*I-1)
      TYB(I)=R(2*I)
      GOTO 50
   20 TXB(I)=UB(I)
      TYB(I)=VB(I)
      UB(I)=R(2*I-1)
      VB(I)=R(2*I)
      GOTO 50
   30 TYB(I)=VB(I)
      TXB(I)=R(2*I-1)
      VB(I)=R(2*I)
      GOTO 50
   40 TXB(I)=UB(I)
      UB(I)=R(2*I-1)
      TYB(I)=R(2*I)
   50 CONTINUE
C
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE UVINTER(XL,YL,XIN,YIN,UB,VB,TXB,TYB,UIN,VIN,N,IN,NL,NB)
C
C
C  This subroutine computes the values of the displacements u and v
C  at the internal points
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN),UB(N),VB(N)
      DIMENSION TXB(N),TYB(N),UIN(IN),VIN(IN)
      DIMENSION NL(NB)
C
C  Compute the values of u and v at the internal points
C
      IF(NB.GT.1) GOTO 5
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
C
    5 DO 10  KK=1,IN
      UIN(KK)=0.
      VIN(KK)=0.
      DO 20  J=1,N
      X0=XIN(KK)
      Y0=YIN(KK)
      X1=XL(J)
      Y1=YL(J)
      X2=XL(J+1)
      Y2=YL(J+1)
      IF(NB.LE.1)GOTO 60
        IF(J.NE.NL(1))GOTO 50
            X2=XL(1)
            Y2=YL(1)
            GOTO 60
   50      DO 30 K=2,NB
          IF(J.NE.NL(K))GOTO 30
            X2=XL(NL(K-1)+1)
            Y2=YL(NL(K-1)+1)
            GOTO 60
   30       CONTINUE
C
   60 IFLAG=2
C
      CALL RLINTG(X0,Y0,X1,Y1,X2,Y2,G11,G12,G22,IFLAG)
      CALL RLINTH(X0,Y0,X1,Y1,X2,Y2,H11,H12,H21,H22)
      UIN(KK)=UIN(KK)-H11*UB(J)-H21*VB(J)+G11*TXB(J)+G12*TYB(J)
   20 VIN(KK)=VIN(KK)-H12*UB(J)-H22*VB(J)+G12*TXB(J)+G22*TYB(J)
   10 CONTINUE
C
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE STRESSIN(XL,YL,XIN,YIN,UB,VB,TXB,TYB,
     1                    SXIN,SYIN,SXYIN,N,IN,NL,NB)
C
C
C  This subroutine computes the stresses at the internal points
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN),UB(N),VB(N)
      DIMENSION TXB(N),TYB(N)
      DIMENSION SXIN(IN),SYIN(IN),SXYIN(IN)
      DIMENSION NL(NB)
C
      IF(NB.GT.1) GOTO 5
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
C
    5 DO 10  KK=1,IN
      SXIN(KK)=0.
      SYIN(KK)=0.
      SXYIN(KK)=0.
      DO 20  J=1,N
      X0=XIN(KK)
      Y0=YIN(KK)
      X1=XL(J)
      Y1=YL(J)
      X2=XL(J+1)
      Y2=YL(J+1)
      IF(NB.LE.1)GOTO 60
        IF(J.NE.NL(1))GOTO 50
            X2=XL(1)
            Y2=YL(1)
            GOTO 60
   50      DO 30 K=2,NB
          IF(J.NE.NL(K))GOTO 30
            X2=XL(NL(K-1)+1)
            Y2=YL(NL(K-1)+1)
            GOTO 60
   30       CONTINUE
   60 CALL SLINTH(X0,Y0,X1,Y1,X2,Y2,RES11,RES21,RES31,RES12,RES22,
     1           RES32,RESB11,RESB21,RESB31,RESB12,RESB22,RESB32)
C
      SXIN(KK)=SXIN(KK)+RES11*TXB(J)+RES12*TYB(J)
     1               -RESB11*UB(J)-RESB12*VB(J)
      SYIN(KK)=SYIN(KK)+RES21*TXB(J)+RES22*TYB(J)
     1               -RESB21*UB(J)-RESB22*VB(J)
   20 SXYIN(KK)=SXYIN(KK)+RES31*TXB(J)+RES32*TYB(J)
     1                 -RESB31*UB(J)-RESB32*VB(J)
   10 CONTINUE
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE SLINTH (X0,Y0,X1,Y1,X2,Y2,RES11,RES21,RES31,RES12,
     1        RES22,RES32,RESB11,RESB21,RESB31,RESB12,RESB22,RESB32)
C
C
C  This subroutine computes the line integrals of the kernels in the
C  integral representations of the stresses
C
C  RA= The distance of the point O to the Gauss integration point 
C      on the boundary element
C
C  WG= The weights of the Gauss integration
C
C  XI= The coordinates of the Gauss integration points in the 
C      interval [-1,1]
C
C  XC,YC=The global coordinates of the Gauss integration points
C
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON EL,GL,PN,ALAMDA
      DIMENSION XC(4),YC(4),XI(4),WG(4)
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      BX=(X2+X1)/2.
      BY=(Y2+Y1)/2.
      SL=DSQRT(AX**2+AY**2)
      ENX=AY/SL
      ENY=-AX/SL
C
C  Compute the line integral
C
      RES11=0.
      RES21=0.
      RES31=0.
      RES12=0.
      RES22=0.
      RES32=0.
      RESB11=0.
      RESB21=0.
      RESB31=0.
      RESB12=0.
      RESB22=0.
      RESB32=0.
C
      A1=-(1.+PN)/(4.*PI)
      A2=(1.-PN)/(1.+PN)
      A3=-2.*GL*A1
      DO 30  I=1,4
      XC(I)=AX*XI(I)+BX
      YC(I)=AY*XI(I)+BY
      DX=XC(I)-X0
      DY=YC(I)-Y0
      RA=DSQRT(DX**2+DY**2)
      COSA=-DX/RA
      SINA=-DY/RA
      RN=COSA*ENX+SINA*ENY
C
      RES11=RES11+(A1/RA)*(A2*COSA+2.*COSA**3)*WG(I)*SL
      RES21=RES21+(A1/RA)*(-A2*COSA+2.*COSA*SINA**2)*WG(I)*SL
      RES31=RES31+(A1/RA)*(A2*SINA+2.*COSA**2*SINA)*WG(I)*SL
      RES12=RES12+(A1/RA)*(-A2*SINA+2.*COSA**2*SINA)*WG(I)*SL
      RES22=RES22+(A1/RA)*(A2*SINA+2.*SINA**3)*WG(I)*SL
      RES32=RES32+(A1/RA)*(A2*COSA+2.*COSA*SINA**2)*WG(I)*SL
C
      RESB11=RESB11+(A3/RA**2)*(2.*COSA*RN*(1.-4.*COSA**2)+
     1                         (2.*COSA**2+1.)*ENX)*WG(I)*SL

      RESB21=RESB21+(A3/RA**2)*(-8.*SINA**2*COSA*RN+2.*COSA*SINA*ENY+
     1                                         ENX)*WG(I)*SL

      RESB12=RESB12+(A3/RA**2)*(-8.*COSA**2*SINA*RN+2.*COSA*SINA*ENX+
     1                                         ENY)*WG(I)*SL

      RESB22=RESB22+(A3/RA**2)*(2.*SINA*RN*(1.-4.*SINA**2)+
     1                         (2.*SINA**2+1.)*ENY)*WG(I)*SL
   30 CONTINUE
      RESB31=RESB12
      RESB32=RESB21
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE STRESSB(XL,YL,UB,VB,TXB,TYB,SXB,SYB,SXYB,N,NL,NB)
C
C
C  This subroutine computes the stress at the boundary 
C  nodal points
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON EL,GL,PN,ALAMDA
      DIMENSION XL(N+1),YL(N+1),UB(N),VB(N),TXB(N),TYB(N)
      DIMENSION SXB(N),SYB(N),SXYB(N),SL(N),ENX(N),ENY(N),A(4,4),B(4)
      DIMENSION NL(NB)
C
      IF(NB.GT.1) GOTO 5
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
C
C  Computation of the element half lengths and direction
C  cosines of the normal vector
C
    5 DO 10 K=1,NB
      IF(K.EQ.1)THEN
      NF=1
      ELSE
      NF=NL(K-1)+1
      ENDIF
      DO 10 I=NF,NL(K)
      IF(I.EQ.NL(K))THEN
      AX=(XL(NF)-XL(NL(K)))/2.
      AY=(YL(NF)-YL(NL(K)))/2.
      SL(I)=DSQRT(AX**2+AY**2)
      ENX(I)=AY/SL(I)
      ENY(I)=-AX/SL(I)
      ELSE
      AX=(XL(I+1)-XL(I))/2.
      AY=(YL(I+1)-YL(I))/2.
      SL(I)=DSQRT(AX**2+AY**2)
      ENX(I)=AY/SL(I)
      ENY(I)=-AX/SL(I)
      ENDIF
   10 CONTINUE
C
C  Computation of the tangential derivatives UT and VT
C
      DO 20 K=1,NB
      IF(K.EQ.1)THEN
      NF=1
      ELSE
      NF=NL(K-1)+1
      ENDIF
      DO 20 I=NF,NL(K)
      IF (I.EQ.NF) THEN
      S1=SL(NL(K))+SL(NF)
      S2=SL(NF)+SL(NF+1)
      U1=UB(NL(K))
      U2=UB(NF)
      U3=UB(NF+1)
      V1=VB(NL(K))
      V2=VB(NF)
      V3=VB(NF+1)
      ELSE IF (I.EQ.NL(K)) THEN
      S1=SL(NL(K)-1)+SL(NL(K))
      S2=SL(NL(K))+SL(NF)
      U1=UB(NL(K)-1)
      U2=UB(NL(K))
      U3=UB(NF)
      V1=VB(NL(K)-1)
      V2=VB(NL(K))
      V3=VB(NF)
      ELSE
      S1=SL(I-1)+SL(I)
      S2=SL(I)+SL(I+1)
      U1=UB(I-1)
      U2=UB(I)
      U3=UB(I+1)
      V1=VB(I-1)
      V2=VB(I)
      V3=VB(I+1)
      ENDIF
C
C  Computation of the boundary stresses
C
      UT=(S1**2*U3-S2**2*U1+(S2**2-S1**2)*U2)
     1      /(S1*S2*(S1+S2))
      VT=(S1**2*V3-S2**2*V1+(S2**2-S1**2)*V2)
     1      /(S1*S2*(S1+S2))
      A(1,1)=(ALAMDA+2.*GL)*ENX(I)
      A(1,2)=GL*ENY(I)
      A(1,3)=GL*ENY(I)
      A(1,4)=ALAMDA*ENX(I)
      A(2,1)=ALAMDA*ENY(I)
      A(2,2)=GL*ENX(I)
      A(2,3)=GL*ENX(I)
      A(2,4)=(ALAMDA+2.*GL)*ENY(I)
      A(3,1)=-ENY(I)
      A(3,2)=ENX(I)
      A(3,3)=0.
      A(3,4)=0.
      A(4,1)=0.
      A(4,2)=0.
      A(4,3)=-ENY(I)
      A(4,4)=ENX(I)
      B(1)=TXB(I)
      B(2)=TYB(I)
      B(3)=UT
      B(4)=VT
      CALL LEQS(A,B,4,LSING)
      SXB(I)=ALAMDA*(B(1)+B(4))+2.*GL*B(1)
      SYB(I)=ALAMDA*(B(1)+B(4))+2.*GL*B(4)
      SXYB(I)=GL*(B(2)+B(3))
   20 CONTINUE
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE OUTPUTEL(XM,YM,UB,VB,TXB,TYB,XIN,YIN,UIN,VIN,
     1                    SXB,SYB,SXYB,SXIN,SYIN,SXYIN,N,IN)
C
C
C  This subroutine prints the results in the outputfile.
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XM(N),YM(N),UB(N),VB(N),TXB(N),TYB(N)
      DIMENSION XIN(N),YIN(N),UIN(IN),VIN(IN)
      DIMENSION SXB(N),SYB(N),SXYB(N)
      DIMENSION SXIN(IN),SYIN(IN),SXYIN(IN)
C
      WRITE(2,100)
  100 FORMAT('',76('*')///'RESULTS'///2X,'BOUNDARY NODES'//'NODE',
     1         5X,'X',11X,'Y',12X,'U',11X,'V',10X,'TXB',9X,'TYB'/)
      DO 10 I=1,N
   10 WRITE(2,200)I,XM(I),YM(I),UB(I),VB(I),TXB(I),TYB(I)
  200 FORMAT(I3,2(1X,E11.5),1X,4(1X,E11.5))
C
      WRITE (2,300)
  300 FORMAT(//,2X,'DISPLACEMENTS AT INTERNAL POINTS'//'  POINT',9X,
     1         'X',14X,'Y',15X,'U',15X,'V'/)
      DO 20 K=1,IN
   20 WRITE(2,400)K,XIN(K),YIN(K),UIN(K),VIN(K)
  400 FORMAT(I5,4(2X,E14.5))
      WRITE(2,600)
  600 FORMAT(//,2X,'STRESSES AT THE BOUNDARY NODAL POINTS'/)
      WRITE(2,700)
  700 FORMAT('NODE',7X,'X',14X,'Y',11X,'SXB',12X,'SYB',10X,'SXYB'/)
      DO 30 I=1,N
   30 WRITE(2,800)I,XM(I),YM(I),SXB(I),SYB(I),SXYB(I)
  800 FORMAT(I3,5(2X,E12.5))
      WRITE (2,900)
  900 FORMAT(//,2X,'STRESSES AT THE INTERNAL POINTS'/)
      WRITE (2,1000)
 1000 FORMAT('NODE',7X,'X',14X,'Y',11X,'SXIN',10X,'SYIN',10X,'SXYIN'/)
      DO 40 I=1,IN
   40 WRITE(2,800)I,XIN(I),YIN(I),SXIN(I),SYIN(I),SXYIN(I)
      WRITE(2,500)
  500 FORMAT(/,'',76('*'))
      RETURN
      END
C
C
C==================================================================
